﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security;
using System.Web;
using System.Web.Security;
using System.Web.UI;

namespace WebApplication1
{
	// *************************************************************************
	// *** WebFormLogin ********************************************************
	// *************************************************************************
	public partial class WebFormLogin : Page
	{
		// ---------------------------------------------------------------------
		internal static readonly Dictionary<string, Tuple<string, MyRole>> Uzivatelia = new Dictionary<string, Tuple<string, MyRole>>()
		{
			{ "uzivatel", new Tuple<string, MyRole>("heslo", MyRole.User) },
			{ "manazer", new Tuple<string, MyRole>("heslo", MyRole.User | MyRole.Manager) },
			{ "admin", new Tuple<string, MyRole>("admin", MyRole.Admin) },
		};

		// ---------------------------------------------------------------------
		// --- WebFormLogin Events ---------------------------------------------
		// ---------------------------------------------------------------------
		protected void Page_Load(object sender, EventArgs e)
		{
			if (!IsPostBack)
				Logout(Context);
		}

		// ---------------------------------------------------------------------
		protected void ButtonLogin_Click(object sender, EventArgs e)
		{
			bool validnyUzivatel =
				Uzivatelia.Keys.Contains(TextBoxLoginName.Text) &&
				Uzivatelia[TextBoxLoginName.Text].Item1 == TextBoxPassword.Text;

			if (!validnyUzivatel)
				throw new SecurityException("Pristup odmietnuty!");

			FormsAuthentication.RedirectFromLoginPage(/*userName*/TextBoxLoginName.Text, /*persitant*/false);
		}

		// ---------------------------------------------------------------------
		protected void ButtonLoginRozneStranky_Click(object sender, EventArgs e)
		{
			bool validnyUzivatel =
				Uzivatelia.Keys.Contains(TextBoxLoginName.Text) &&
				Uzivatelia[TextBoxLoginName.Text].Item1 == TextBoxPassword.Text;

			if (!validnyUzivatel)
				throw new SecurityException("Pristup odmietnuty!");

			// Autorizaujeme uzivatela
			FormsAuthentication.SetAuthCookie(/*userName*/TextBoxLoginName.Text, /*persitantCookies*/false);

			// Manualne ho redirectujeme
			Response.Redirect(TextBoxLoginName.Text == "admin" ? "WebForm2.aspx" : "WebForm1.aspx");
		}

		// ---------------------------------------------------------------------
		protected void ButtonLoginManualny_Click(object sender, EventArgs e)
		{
			bool validnyUzivatel =
				Uzivatelia.Keys.Contains(TextBoxLoginName.Text) &&
				Uzivatelia[TextBoxLoginName.Text].Item1 == TextBoxPassword.Text;

			if (!validnyUzivatel)
				throw new SecurityException("Pristup odmietnuty!");

			var ticket = new FormsAuthenticationTicket(
				/*version*/3,
				TextBoxLoginName.Text,
				/*start*/DateTime.Now,
				/*expire*/DateTime.Now.AddMinutes(120),
				/*persistantCookies*/false,
				/*data*/string.Empty);

			string hash = FormsAuthentication.Encrypt(ticket);

			var authCookie = new HttpCookie(FormsAuthentication.FormsCookieName, hash);
			authCookie.Domain = FormsAuthentication.CookieDomain;

			Context.Response.Cookies.Add(authCookie);

			// Manualne ho redirectujeme
			Response.Redirect(TextBoxLoginName.Text == "admin" ? "WebForm2.aspx" : "WebForm1.aspx");
		}

		// ---------------------------------------------------------------------
		// --- WebFormLogin Methods --------------------------------------------
		// ---------------------------------------------------------------------
		public static void Logout(HttpContext context, bool restoreSession = true)
		{
			// Clear member access informations
			FormsAuthentication.SignOut();

			// FF caching issue
			var formCookie = new HttpCookie(FormsAuthentication.FormsCookieName, /*value*/string.Empty);
			formCookie.Expires = DateTime.Now.AddYears(-1);
			context.Response.Cookies.Add(formCookie);

			// Clear session
			context.Session.RemoveAll();
			context.Session.Clear();

			if (restoreSession)
			{
				context.Session.Abandon();

				// FF caching issue
				var sessionCookie = new HttpCookie("ASP.NET_SessionId", /*value*/string.Empty);
				sessionCookie.Expires = DateTime.Now.AddYears(-1);
				context.Response.Cookies.Add(sessionCookie);
			} // if
		}
	} // public partial class WebFormLogin

} // namespace WebApplication1